# encoding: utf-8
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.

module Azure::DigitalTwins::Mgmt::V2020_10_31
  #
  # Azure Digital Twins Client for managing DigitalTwinsInstance
  #
  class DigitalTwinsEndpoint
    include MsRestAzure

    #
    # Creates and initializes a new instance of the DigitalTwinsEndpoint class.
    # @param client service class for accessing basic functionality.
    #
    def initialize(client)
      @client = client
    end

    # @return [AzureDigitalTwinsManagementClient] reference to the AzureDigitalTwinsManagementClient
    attr_reader :client

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Array<DigitalTwinsEndpointResource>] operation results.
    #
    def list(resource_group_name, resource_name, custom_headers:nil)
      first_page = list_as_lazy(resource_group_name, resource_name, custom_headers:custom_headers)
      first_page.get_all_items
    end

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_with_http_info(resource_group_name, resource_name, custom_headers:nil)
      list_async(resource_group_name, resource_name, custom_headers:custom_headers).value!
    end

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_async(resource_group_name, resource_name, custom_headers:nil)
      fail ArgumentError, "'@client.api_version' should satisfy the constraint - 'MinLength': '10'" if !@client.api_version.nil? && @client.api_version.length < 10
      @client.api_version = '2020-10-31'
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MaxLength': '64'" if !resource_group_name.nil? && resource_group_name.length > 64
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MaxLength': '63'" if !resource_name.nil? && resource_name.length > 63
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MinLength': '3'" if !resource_name.nil? && resource_name.length < 3
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'Pattern': '^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$'" if !resource_name.nil? && resource_name.match(Regexp.new('^^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DigitalTwins/digitalTwinsInstances/{resourceName}/endpoints'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'resourceName' => resource_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResourceListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get DigitalTwinsInstances Endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResource] operation results.
    #
    def get(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      response = get_async(resource_group_name, resource_name, endpoint_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get DigitalTwinsInstances Endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def get_with_http_info(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      get_async(resource_group_name, resource_name, endpoint_name, custom_headers:custom_headers).value!
    end

    #
    # Get DigitalTwinsInstances Endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def get_async(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      fail ArgumentError, "'@client.api_version' should satisfy the constraint - 'MinLength': '10'" if !@client.api_version.nil? && @client.api_version.length < 10
      @client.api_version = '2020-10-31'
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MaxLength': '64'" if !resource_group_name.nil? && resource_group_name.length > 64
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MaxLength': '63'" if !resource_name.nil? && resource_name.length > 63
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MinLength': '3'" if !resource_name.nil? && resource_name.length < 3
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'Pattern': '^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$'" if !resource_name.nil? && resource_name.match(Regexp.new('^^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$$')).nil?
      fail ArgumentError, 'endpoint_name is nil' if endpoint_name.nil?
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'MaxLength': '49'" if !endpoint_name.nil? && endpoint_name.length > 49
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'MinLength': '2'" if !endpoint_name.nil? && endpoint_name.length < 2
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'Pattern': '^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$'" if !endpoint_name.nil? && endpoint_name.match(Regexp.new('^^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DigitalTwins/digitalTwinsInstances/{resourceName}/endpoints/{endpointName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'resourceName' => resource_name,'endpointName' => endpoint_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Create or update DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param endpoint_description [DigitalTwinsEndpointResource] The
    # DigitalTwinsInstance endpoint metadata and security metadata.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResource] operation results.
    #
    def create_or_update(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:nil)
      response = create_or_update_async(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param endpoint_description [DigitalTwinsEndpointResource] The
    # DigitalTwinsInstance endpoint metadata and security metadata.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def create_or_update_async(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:nil)
      # Send request
      promise = begin_create_or_update_async(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Delete a DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResource] operation results.
    #
    def delete(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      response = delete_async(resource_group_name, resource_name, endpoint_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [Concurrent::Promise] promise which provides async access to http
    # response.
    #
    def delete_async(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      # Send request
      promise = begin_delete_async(resource_group_name, resource_name, endpoint_name, custom_headers:custom_headers)

      promise = promise.then do |response|
        # Defining deserialization method.
        deserialize_method = lambda do |parsed_response|
          result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
          parsed_response = @client.deserialize(result_mapper, parsed_response)
        end

        # Waiting for response.
        @client.get_long_running_operation_result(response, deserialize_method)
      end

      promise
    end

    #
    # Create or update DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param endpoint_description [DigitalTwinsEndpointResource] The
    # DigitalTwinsInstance endpoint metadata and security metadata.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResource] operation results.
    #
    def begin_create_or_update(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:nil)
      response = begin_create_or_update_async(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Create or update DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param endpoint_description [DigitalTwinsEndpointResource] The
    # DigitalTwinsInstance endpoint metadata and security metadata.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_create_or_update_with_http_info(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:nil)
      begin_create_or_update_async(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:custom_headers).value!
    end

    #
    # Create or update DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param endpoint_description [DigitalTwinsEndpointResource] The
    # DigitalTwinsInstance endpoint metadata and security metadata.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_create_or_update_async(resource_group_name, resource_name, endpoint_name, endpoint_description, custom_headers:nil)
      fail ArgumentError, "'@client.api_version' should satisfy the constraint - 'MinLength': '10'" if !@client.api_version.nil? && @client.api_version.length < 10
      @client.api_version = '2020-10-31'
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MaxLength': '64'" if !resource_group_name.nil? && resource_group_name.length > 64
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MaxLength': '63'" if !resource_name.nil? && resource_name.length > 63
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MinLength': '3'" if !resource_name.nil? && resource_name.length < 3
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'Pattern': '^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$'" if !resource_name.nil? && resource_name.match(Regexp.new('^^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$$')).nil?
      fail ArgumentError, 'endpoint_name is nil' if endpoint_name.nil?
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'MaxLength': '49'" if !endpoint_name.nil? && endpoint_name.length > 49
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'MinLength': '2'" if !endpoint_name.nil? && endpoint_name.length < 2
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'Pattern': '^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$'" if !endpoint_name.nil? && endpoint_name.match(Regexp.new('^^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$$')).nil?
      fail ArgumentError, 'endpoint_description is nil' if endpoint_description.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?

      # Serialize Request
      request_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
      request_content = @client.serialize(request_mapper,  endpoint_description)
      request_content = request_content != nil ? JSON.generate(request_content, quirks_mode: true) : nil

      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DigitalTwins/digitalTwinsInstances/{resourceName}/endpoints/{endpointName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'resourceName' => resource_name,'endpointName' => endpoint_name},
          query_params: {'api-version' => @client.api_version},
          body: request_content,
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:put, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 201
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 201
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Delete a DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResource] operation results.
    #
    def begin_delete(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      response = begin_delete_async(resource_group_name, resource_name, endpoint_name, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Delete a DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def begin_delete_with_http_info(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      begin_delete_async(resource_group_name, resource_name, endpoint_name, custom_headers:custom_headers).value!
    end

    #
    # Delete a DigitalTwinsInstance endpoint.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param endpoint_name [String] Name of Endpoint Resource.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def begin_delete_async(resource_group_name, resource_name, endpoint_name, custom_headers:nil)
      fail ArgumentError, "'@client.api_version' should satisfy the constraint - 'MinLength': '10'" if !@client.api_version.nil? && @client.api_version.length < 10
      @client.api_version = '2020-10-31'
      fail ArgumentError, '@client.subscription_id is nil' if @client.subscription_id.nil?
      fail ArgumentError, 'resource_group_name is nil' if resource_group_name.nil?
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MaxLength': '64'" if !resource_group_name.nil? && resource_group_name.length > 64
      fail ArgumentError, "'resource_group_name' should satisfy the constraint - 'MinLength': '1'" if !resource_group_name.nil? && resource_group_name.length < 1
      fail ArgumentError, 'resource_name is nil' if resource_name.nil?
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MaxLength': '63'" if !resource_name.nil? && resource_name.length > 63
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'MinLength': '3'" if !resource_name.nil? && resource_name.length < 3
      fail ArgumentError, "'resource_name' should satisfy the constraint - 'Pattern': '^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$'" if !resource_name.nil? && resource_name.match(Regexp.new('^^(?!-)[A-Za-z0-9-]{3,63}(?<!-)$$')).nil?
      fail ArgumentError, 'endpoint_name is nil' if endpoint_name.nil?
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'MaxLength': '49'" if !endpoint_name.nil? && endpoint_name.length > 49
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'MinLength': '2'" if !endpoint_name.nil? && endpoint_name.length < 2
      fail ArgumentError, "'endpoint_name' should satisfy the constraint - 'Pattern': '^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$'" if !endpoint_name.nil? && endpoint_name.match(Regexp.new('^^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$$')).nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = 'subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.DigitalTwins/digitalTwinsInstances/{resourceName}/endpoints/{endpointName}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          path_params: {'subscriptionId' => @client.subscription_id,'resourceGroupName' => resource_group_name,'resourceName' => resource_name,'endpointName' => endpoint_name},
          query_params: {'api-version' => @client.api_version},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:delete, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200 || status_code == 202 || status_code == 204
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end
        # Deserialize Response
        if status_code == 202
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResource.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResourceListResult] operation results.
    #
    def list_next(next_page_link, custom_headers:nil)
      response = list_next_async(next_page_link, custom_headers:custom_headers).value!
      response.body unless response.nil?
    end

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [MsRestAzure::AzureOperationResponse] HTTP response information.
    #
    def list_next_with_http_info(next_page_link, custom_headers:nil)
      list_next_async(next_page_link, custom_headers:custom_headers).value!
    end

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param next_page_link [String] The NextLink from the previous successful call
    # to List operation.
    # @param [Hash{String => String}] A hash of custom headers that will be added
    # to the HTTP request.
    #
    # @return [Concurrent::Promise] Promise object which holds the HTTP response.
    #
    def list_next_async(next_page_link, custom_headers:nil)
      fail ArgumentError, 'next_page_link is nil' if next_page_link.nil?


      request_headers = {}
      request_headers['Content-Type'] = 'application/json; charset=utf-8'

      # Set Headers
      request_headers['x-ms-client-request-id'] = SecureRandom.uuid
      request_headers['accept-language'] = @client.accept_language unless @client.accept_language.nil?
      path_template = '{nextLink}'

      request_url = @base_url || @client.base_url

      options = {
          middlewares: [[MsRest::RetryPolicyMiddleware, times: 3, retry: 0.02], [:cookie_jar]],
          skip_encoding_path_params: {'nextLink' => next_page_link},
          headers: request_headers.merge(custom_headers || {}),
          base_url: request_url
      }
      promise = @client.make_request_async(:get, path_template, options)

      promise = promise.then do |result|
        http_response = result.response
        status_code = http_response.status
        response_content = http_response.body
        unless status_code == 200
          error_model = JSON.load(response_content)
          fail MsRest::HttpOperationError.new(result.request, http_response, error_model)
        end

        result.request_id = http_response['x-ms-request-id'] unless http_response['x-ms-request-id'].nil?
        result.correlation_request_id = http_response['x-ms-correlation-request-id'] unless http_response['x-ms-correlation-request-id'].nil?
        result.client_request_id = http_response['x-ms-client-request-id'] unless http_response['x-ms-client-request-id'].nil?
        # Deserialize Response
        if status_code == 200
          begin
            parsed_response = response_content.to_s.empty? ? nil : JSON.load(response_content)
            result_mapper = Azure::DigitalTwins::Mgmt::V2020_10_31::Models::DigitalTwinsEndpointResourceListResult.mapper()
            result.body = @client.deserialize(result_mapper, parsed_response)
          rescue Exception => e
            fail MsRest::DeserializationError.new('Error occurred in deserializing the response', e.message, e.backtrace, result)
          end
        end

        result
      end

      promise.execute
    end

    #
    # Get DigitalTwinsInstance Endpoints.
    #
    # @param resource_group_name [String] The name of the resource group that
    # contains the DigitalTwinsInstance.
    # @param resource_name [String] The name of the DigitalTwinsInstance.
    # @param custom_headers [Hash{String => String}] A hash of custom headers that
    # will be added to the HTTP request.
    #
    # @return [DigitalTwinsEndpointResourceListResult] which provide lazy access to
    # pages of the response.
    #
    def list_as_lazy(resource_group_name, resource_name, custom_headers:nil)
      response = list_async(resource_group_name, resource_name, custom_headers:custom_headers).value!
      unless response.nil?
        page = response.body
        page.next_method = Proc.new do |next_page_link|
          list_next_async(next_page_link, custom_headers:custom_headers)
        end
        page
      end
    end

  end
end
